;last updated November 2001
; -- Aaron Curtis

;***************
;*** equates ***
;***************

dt_space_low equ $4100
prog_space_low equ $4800
num_dts_low equ $5800
num_progs_low equ $5801

dt_space equ $8100
dt_pri_table equ $8700
dt_primary equ $8800
prog_space equ $8800
num_dts equ $9800
num_progs equ $9801

temp_space equ $f400

;************************
;*** primary routines ***
;************************

yas_writeback:
	;** writes back the shell **
	;no inputs
	;no outputs
	;destroys hl,bc,de,af

	ld hl,_asapvar
	rst 20h
	rst 10h
	ret c
	ex de,hl
	ld a,b
	ld de,-$4000
	call yt_loadports
	ld c,(hl)
	inc hl
	ld b,(hl)
	inc hl
	inc hl
	inc hl
	dec bc
	dec bc
	ex de,hl
	ld hl,_asm_exec_ram
	ldir
	jr yt_jpstd


yas_readgroup:
	;** reads free vat byte **
	;input: hl->file in table
	;output: a=contents of free byte
	;destroys hl,de,af

	call yt_groupcode
	ld a,(hl)
	jr yt_jpstd


yas_writegroup:
	;** writes free vat byte **
	;input: hl->file in table
	;		a=data to write
	;no output
	;destroys hl,de,f

	call yt_groupcode
	ld (hl),a
yt_jpstd:
	jp yas_stdports


yas_execute:
	;** executes file **
	;input: hl->file in table
	;no outputs
	;destroys all

#define yhigh(x) $f400+x-yt_highjump

	push hl
	call yt_locatedt
	push hl
	ld de,dt_space+5
	add hl,de
	call yt_loadeda
	call yt_setports
	ld bc,4
	add hl,bc
	call $0033
	ld (yt_execjp),hl
	call yas_stdports

	ld hl,yt_highjump			;_asm_exec_ram will be overwritten
	ld de,$f400					;so copy this routine to high memory
	ld bc,yas_refresh-yt_highjump
	ldir
	jp yhigh(yt_highjump)
yt_highjump	:

	ld de,(_OPS)
	set 7,d						;*grumble*
	ld hl,_asapvar+9
	ld bc,10
	lddr						;push current program onto OP stack
	ld (yhigh(yt_opload)),de	;save location of program name
	ld hl,yhigh(yt_oplast)
	ld bc,yas_stdports-yt_opcode
	lddr						;push restoration code onto OP stack
	ex de,hl
	res 7,h
	ld (_OPS),hl				;save new OP stack pointer
	set 7,h
	inc hl
	pop de
	ex (sp),hl

	push hl
	ld hl,dt_space
	add hl,de
	call $0033					;find vat address of file containing dt
	call _ram_page_7
	ld de,_OP1+1
	call yt_vattode 			;load file name into _OP1 (not destroyed yet...)
	call _asap_to_ram			;copy file to _asm_exec_ram
	ld de,_asapvar
	call _movop1to
	call yhigh(yas_stdports)

	pop hl
	ld bc,5
	add hl,bc
	rst 20h						;put program to execute in _OP1

	ld de,_ram_page_1
	push de						;new program will return to page 1
yt_execjp equ $+1
	jp _asm_exec_ram

yt_opcode:
yt_opload equ $ +1
	ld hl,0						;this will point to the name of the shell
	inc hl						;inc hl because the lddr command leaves it
								; pointing one byte too low
	rst 20h						;put it in _OP1
	dec hl						;rst 20h ouputs the next byte after the name
	res 7,h
	ld (_OPS),hl				;restore old OP stack pointer
	ld de,_asapvar
	call _movop1to
	ld de,_ram_page_1			;_asap_to_ram affects the pages
	push de
	jp _asap_to_ram				;load (_OP1) to _asm_exec_ram (thanks Josh)
yt_oplast equ $-1


yas_stdports:
	;sets standard rom and ram pages
	;no inputs
	;no outputs
	;destroys none

	push af
    ld a,$0d
    out (5),a
	pop af
    jp _RAM_PAGE_1


yas_refresh:
	;** builds program table **
	;no inputs
	;no outputs
	;destorys all

	;* initialize some memory areas *
	call yas_stdports
	ld hl,prog_space_low
    ld (yt_ldhllist),hl
	ld h,l
	ld (num_dts),hl

	ld h,dt_pri_table/256
	ld de,dt_pri_table+1
	ld bc,255
	ld (hl),l
	ldir

	;* first search for datatypes *
    ld de,yt_dtsearch			;set routine to find datatypes
    call yt_search              

	ld a,(num_dts)				;return if no datatypes
	or a
	ret z
	ld (yt_lddts),a

	;* sort datatypes by priority *
	ld bc,dt_pri_table+7
	ld hl,dt_primary+(7*2*256)
	ld de,dt_space
sortloop:
		push hl
		ld a,(bc)
		add a,a
		jr z,nextlevel
			push bc
			ld b,0	
			rl b
			ld c,a
			ldir
			pop bc
nextlevel:
		pop hl
		dec h
		dec h
		dec c
		jp p,sortloop

	;* next search for files *
    ld de,yt_filesearch			;set routine to find files
    jp yt_search


yas_icon:
	;input:		hl->program table entry
	;output:	hl->bitmap data
	;			b=width (bytes)
	;			c=height (lines)
	;			or c set if no icon
	;destroys all

	ld c,14
	jr yt_retrievecode


yas_comment:
	;** loads comment **
	;input:		hl->progam table entry
	;output:	hl->null-terminated comment
	;			or c set if no comment
	;destroys all

    ld c,10
yt_retrievecode:
    push hl
    call yt_getdt
    ld de,temp_space
    ldir
    pop hl
    call yas_stdports
    ld c,5
    add hl,bc
    rst 20h
	rst 10h
	ret c
	ld a,b
	ex de,hl
	call _load_ram_ahl
	inc hl
	inc hl
	bit 6,h
	jr z,yt_jptemp
		inc a
		out (6),a
		ld h,$80
yt_jptemp:
    jp temp_space


;********************
;*** sub-routines ***
;********************

yt_search:
	;** search routine **
	;inputs: de->code used in search
	;other inputs vary with the routine...

	ld (yt_searchcall),de
    call yt_stdports7
    ld de,$bfff          		;de=top of the vat
yt_searchloop:
	    push de           		;save vat pointer
yt_searchcall equ $+1
	    call 0 					;examine file using varying code
  		pop de					;restore vat pointer
   		call yt_stdports7
    	push hl					;preserve hl
    	ld hl,-5
    	add hl,de				;move to name length
    	ld d,0
    	ld e,(hl)   
    	sbc hl,de              	;c is set, so move past name
    	ex de,hl
    	ld hl,($d298)
    	sbc hl,de				;c is reset; see if we've reached the end
    	pop hl
    	jr c,yt_searchloop		;continue if not at the end
    	jp yas_stdports

yt_dtsearch:
	;** code to examine possible datatypes **
	;inputs: 	8 bytes at dt_pri_table set to 0
	;			de->file in vat

	call yt_checkfordt
	ret nz
	ld a,(hl)				;first byte is the number of included dt's
	or a
	ret z					;return if there were none
	inc hl
yt_DTsearchloop:
		push af
		push hl
			ld e,(hl)
			inc hl
			ld d,(hl)				;get address of DT info
			call yt_stripaddress
			ld c,(hl)				;get priority byte (0=lowest)
			inc hl
			ld a,(hl)				;get OS type byte
			push af
			ex de,hl
			ld a,$41				;set ram page 1
			out (6),a
				
			ld b,dt_pri_table/256
			ld a,(bc)
			ld l,a
			add a,3
			ld (bc),a

			ld a,dt_primary/512
			add a,c
			ld h,a
			add hl,hl

yt_filetop equ $+1
			ld bc,0
			ld (hl),c				;first two are the vat pointer
			inc hl
			ld (hl),b
			inc hl
			in a,(5)
			ld (hl),a				;then the page of the data
			inc hl
			ld (hl),d				;pointer to the data (reversed)
			inc hl
			ld (hl),e
			inc hl
			pop af
			ld (hl),a				;OS type byte
			ld hl,num_dts
			inc (hl)
		in a,(5)
		inc a
		out (6),a
		pop hl
		pop af
		inc hl
		inc hl
		dec a
		jr nz,yt_DTsearchloop
	ret

yt_checkfordt:
	;opens a file and checks for the appropriate
	;header
	;returns z if header was good
	;returns de->include information (if it exists)
	;inputs: de->file in vat

	ex de,hl			;load actual data
	ld (yt_filetop),hl
	call yt_loadeda
	ld de,-$4000+4
	call yt_loadports
	ld (yt_filestart),hl
	ld a,(hl)
	or a				;check for nop
	ret nz
	inc hl
	ld a,(hl)
	cp $c3				;check for jp
	ret nz
	inc hl
	inc hl
	inc hl
	ld a,(hl)
	cp 2				;check version number (.dw 2)
	ret c				;nz
	inc hl
	ld a,(hl)
	or a
	ret nz
	ld de,5
	add hl,de			;file is good, so get the include data
	ld e,(hl)
	inc hl
	ld a,(hl)
	ld d,a
	or e
	cp 1
	ret c				;ret nz if 0
	xor a
	jp yt_stripaddress
	

yt_filesearch:
	;** routine to examine possible files **
	;inputs:	de->file in vat

    ld hl,dt_space_low-1
    ld c,0                 		;starting on dt #0
yt_lddts equ $+1
    ld b,0						;ld b with the number of datatypes
yt_checktypes:
    push bc                 	;save counter
    ld a,c
    ld (yt_ldtypeathl),a	   	;load the appropriate dt #
    inc hl                  	;move to next entry in dt table
    inc hl
    inc hl
    inc hl
	inc hl
	inc hl
    call yt_stdports7   		;on the vat
    ld a,$41
    out (5),a
    ld a,(de)
    cp (hl)
    jp nz,yt_endchecktypes   	;if types don't match, move to the next dt (done here for speed)
    
    push de                		;save dt pointer and vat entry
    push hl
    push de
    ld bc,6
    call yt_setupcopy       	;load the length and address of the detection code  
    ld de,temp_space
    ldir                        ;copy into temp space
    call yt_stdports7           ;to the call table and vat
    pop hl                      ;get vat entry back
    push hl                     ;save vat entry again
    call yt_loadeda             ;get abs address in ahl
    pop de                      ;get vat entry back in de
    call temp_space	            ;input: ahl points to the start of the program, de to the vat entry
    pop hl                      ;output: c if not the right type, nc if ok
    pop de
    jr c,yt_endchecktypes
    push de                  	;save the vat entry
	ld a,$41
	out (5),a
    call yt_loadeda           	;get address of the dt
    call yt_setports
    inc hl
    ld c,(hl)                	;get 3 byte description
    inc hl
    ld b,(hl)
    inc hl
    ld e,(hl)
	ld a,$41
	out (5),a
	ld a,e
    pop de                      ;get the vat entry back
yt_ldhllist equ $+1				;self modifying code here
    ld hl,prog_space_low   		;now hl points to somewhere in the program list
    ld (hl),e                   ;first 2 bytes are a pointer to the vat entry
    inc l                       ;all entries are an even 16 bytes
    ld (hl),d
    inc l
    ld (hl),c                   ;next 3 are a short description
    inc l
    ld (hl),b
    inc l
    ld (hl),a
    inc l
yt_ldtypeathl equ $+1
    ld (hl),0                   ;next 1 is the dt number
    inc l
    ex de,hl
    ld a,$47
    out (6),a                   ;page 7 in the upper area
	push de

	call yt_vattode
    xor a                       ;then a 0
    ld (de),a

    ld b,5
    ld hl,yt_blacklist
yt_evilnames:
    ld e,(hl)
    inc hl
    ld d,(hl)
	inc hl
    ex (sp),hl
    push hl
    ex de,hl
    ld c,(hl)
yt_checknameloop:
	    inc de
    	inc hl
    	ld a,(de)
    	cp (hl)
    	jr nz,yt_donecheck
		dec c
    	jr nz,yt_checknameloop
    pop hl
	pop hl
	pop bc
    ret
yt_donecheck:
    pop hl
    ex (sp),hl
    djnz yt_evilnames
    pop hl

    ld c,10
    add hl,bc
    ld (yt_ldhllist),hl
    ld hl,num_progs_low
    inc (hl)
    pop bc
    ret
yt_endchecktypes:
    pop bc
    inc c
    dec b
    jp nz,yt_checktypes
    ret



;******************************
;*** miscellaneous routines ***
;******************************

yt_getdt:
	call yt_locatedt
    ld de,dt_space+5
    add hl,de
yt_setupcopy:
    call yt_loadeda
    call yt_setports
    add hl,bc
    ld c,(hl)
    inc hl
    ld b,(hl)
    inc hl
    ld e,(hl)
    inc hl
    ld d,(hl)
    add hl,de
    ret

yt_locatedt:
	ld de,5
	ld b,d
	add hl,de
	ld l,(hl)
	ld h,d
	add hl,hl
	ld d,h
	ld e,l
	add hl,hl
	add hl,de
	ret

yt_stripaddress:		;labels in the program will be .org'd _asm_exec_ram
	ld hl,$10000-$d748	;so this call cuts that off and finds the
	add hl,de			;"real" address
yt_filestart equ $+1	;inputs: de=address in _asm_exec_ram
	ld de,0
	add hl,de			;outputs: de=address in memory
	ret

yt_loadeda:
    dec hl
    ld e,(hl)
    dec hl
    ld d,(hl)
    dec hl
    ld a,(hl)
    ex de,hl
    ret

yt_loadports:
	call _load_ram_ahl		;preserves de
    add hl,de
yt_setports:
    out (5),a
    inc a
    out (6),a
    ret

yt_groupcode:
    ld e,(hl)
    inc hl
    ld d,(hl)
    ex de,hl
    ld de,-4
    add hl,de

yt_stdports7:
    call yas_stdports
    jp _RAM_PAGE_7

yt_vattode
    ld bc,-5
    add hl,bc
    ld b,(hl)
    inc b
yt_writename:
    ld a,(hl)                   ;then the name length
    ld (de),a                   ;followed by the name
    dec hl
    inc e
    djnz yt_writename
	ret


;************
;*** data ***
;************

yt_blacklist:
    .dw yt_number
    .dw yt_exclam
    .dw yt_penglev
    .dw yt_sqrt
    .dw _asapvar+1			;currently running program

yt_sqrt:
    .db 1,$10
yt_number:
    .db 1,"#"               ;these things are ANNOYING! (#=inc hl)
yt_penglev:
    .db 5,"ZPWLD"
yt_exclam:
	.db 1,"!"
